﻿//-----------------------------------------------------------------------
// <copyright file="CleaningService.cs" company="FastNET Services">
//     Copyright (c) 2007-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System.Collections.Generic;
using FastNET.CleanAct.Client.Interfaces.Jobs;
using FastNET.CleanAct.Client.Interfaces.Services;
using FastNET.CleanAct.Client.Interfaces.TaskProcessing.Cleaners;
using FastNET.CleanAct.Client.Interfaces.Tasks;
using FastNET.CleanAct.Client.TaskProcessing.Basic.Cleaners;

#endregion

namespace FastNET.CleanAct.Client.ClientEngine.Managers.Processing.Services
{
    /// <summary>
    /// Service class used for performing cleaning activities
    /// </summary>
    internal class CleaningService : IService
    {
        #region Variables

        private readonly List<ICleaner> _cleaners;

        #endregion

        #region CTor/DTor

        /// <summary>
        /// Initializes a new instance of the CleaningService class
        /// </summary>
        public CleaningService()
        {
            _cleaners = new List<ICleaner>
                        {
                            new FileSystemCleaner(),
                            new RegistryCleaner()
                        };
        }

        #endregion

        #region Public Collection Properties

        /// <summary>
        /// Gets or sets the list of all jobs that can be processed by the service
        /// </summary>
        public List<IJob> JobList { get; set; }

        #endregion

        #region Public Processing Methods

        /// <summary>
        /// Tells the service to process the list of jobs
        /// </summary>
        public void Process()
        {
            foreach (ICleaner cleaner in _cleaners)
            {
                foreach (IJob job in JobList)
                {
                    foreach (ITask task in job.TaskList)
                    {
                        if (!cleaner.CompatibleTaskTypes.Contains(task.GetType()))
                        {
                            continue;
                        }

                        job.ProcessedBy.Add(string.Format("{0} - {1}", cleaner.ProcessorName, cleaner.ProcessorDescription));
                        cleaner.ProcessTask(task);
                    }
                }
            }
        }

        #endregion
    }
}
