﻿//-----------------------------------------------------------------------
// <copyright file="DynamicSetting.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Data;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.Client;
#endregion

namespace FastNET.CleanAct.Client.CleanEngineData.Entities
{
    /// <summary>
    /// Dynamic Setting Entity Data Class
    /// </summary>
    internal partial class DynamicSetting : IDynamicSettingData
    {
        /// <summary>
        /// Initializes a new instance of the DynamicSetting class
        /// </summary>
        public DynamicSetting()
        {
            SettingID = Guid.NewGuid();
        }

        #region State Management

        /// <summary>
        /// Gets a value indicating whether or not this instance has been modified
        /// </summary>
        public bool IsDirty
        {
            get
            {
                if (IsNew)
                {
                    return true;
                }

                return EntityState == EntityState.Modified;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not this instance is new
        /// </summary>
        public bool IsNew
        {
            get
            {
                // Before it is attached if the key is null then we are new
                if (EntityState == EntityState.Detached)
                {
                    if (EntityKey == null)
                    {
                        return true;
                    }
                }

                return EntityState == EntityState.Added;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not this instance has been deleted
        /// </summary>
        public bool IsDeleted
        {
            get
            {
                return EntityState == EntityState.Deleted;
            }
        }

        #endregion

        #region Identity Management

        /// <summary>
        /// Gets an Identity based reference for this item
        /// </summary>
        public Identity ID
        {
            get { return SettingID; }
        }

        #endregion

        #region ID Reference Properties

        /// <summary>
        /// Gets or sets the associated Profile ID for this setting
        /// </summary>
        public Identity? ProfileID
        {
            get
            {
                return UserProfileReference.EntityKey == null ? (Identity?)null : (Guid)UserProfileReference.EntityKey.EntityKeyValues[0].Value;
            }

            set
            {
                if (value.HasValue)
                {
                    if (!value.Value.CanFitInGuid())
                    {
                        throw new InvalidCastException("Identity is not of type Guid ");
                    }

                    UserProfileReference.EntityKey = new EntityKey("EntityDataAdapter.UserProfiles", "ProfileID", (Guid)value.Value);
                }
                else
                {
                    UserProfileReference.EntityKey = null;
                }
            }
        }

        #endregion
    }
}