﻿//-----------------------------------------------------------------------
// <copyright file="SQLDataProvider.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using System.Collections.Generic;
using System.Data.EntityClient;
using FastNET.CleanAct.Client.CleanEngineData.Repositories;
using FastNET.CleanAct.Common.Repository.Interfaces.Providers;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.Base;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.Client;

#endregion

namespace FastNET.CleanAct.Client.CleanEngineData
{
    /// <summary>
    /// Linq based SQLDataProvider for managing repositories and datasources
    /// </summary>
    public class SQLDataProvider : IDataProvider
    {
        #region Fields

        private static readonly Dictionary<Type, IRepository> _repositories;
        private static EntityConnection _connection;

        #endregion

        #region CTor/DTor

        /// <summary>
        /// Initializes static members of the SQLDataProvider class
        /// </summary>
        static SQLDataProvider()
        {
            // Initialize variables
            _repositories = new Dictionary<Type, IRepository>();

            // Create connection
            CreateConnection();

            // Build Repository Library
            _repositories.Add(typeof(IUserProfileRepository), new UserProfileRepository(_connection));
            _repositories.Add(typeof(IDynamicSettingRepository), new DynamicSettingRepository(_connection));
        }

        #endregion

        #region Retrievals

        /// <summary>
        /// Gets the requested repository based on the TypeParam
        /// </summary>
        /// <typeparam name="T">RepositoryType to retrieve</typeparam>
        /// <returns>IRepositoryBase with the requested repository or null if none was found</returns>
        public T GetRepository<T>() where T : IRepository
        {
            if (_repositories.ContainsKey(typeof(T)))
            {
                return (T)_repositories[typeof(T)];
            }

            throw new InvalidOperationException(string.Format("The repository of type {0} is unsupported by this provider", typeof(T)));
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Creates the general connection for connecting to the data store
        /// </summary>
        private static void CreateConnection()
        {
            EntityConnectionStringBuilder builder = new EntityConnectionStringBuilder
                                                    {
                                                        Metadata = "metadata=res://*/Entities.Model.csdl|res://*/Entities.Model.ssdl|res://*/Entities.Model.msl;",
                                                        Provider = "System.Data.SqlServerCe.3.5",
                                                        ProviderConnectionString = Properties.Settings.Default.ClientDataConnectionString
                                                    };

            _connection = new EntityConnection(builder.ConnectionString);

            _connection.Open();
        }

        #endregion
    }
}