﻿//-----------------------------------------------------------------------
// <copyright file="ProcessingServiceElement.cs" company="FastNET Services">
//     Copyright (c) 2007-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Configuration;
#endregion

namespace FastNET.CleanAct.Client.Configuration.ProcessingServices
{
    /// <summary>
    /// ConfigurationElement for a processing service
    /// </summary>
    public class ProcessingServiceElement : ConfigurationElement
    {
        #region Public Collections

        /// <summary>
        /// Gets a list of all the groups associated with the service
        /// </summary>
        [ConfigurationProperty("groups", IsDefaultCollection = false)]
        [ConfigurationCollection(typeof(ProcessingGroupElementCollection), AddItemName = "add", ClearItemsName = "clear", RemoveItemName = "remove")]
        public ProcessingGroupElementCollection Groups
        {
            get
            {
                return (ProcessingGroupElementCollection)this["groups"] ?? new ProcessingGroupElementCollection();
            }
        }

        #endregion

        /// <summary>
        /// Gets or sets the name for the processing service
        /// </summary>
        [ConfigurationProperty("name", IsRequired = true, IsKey = true)]
        public string Name
        {
            get
            {
                return this["name"] as string;
            }

            set
            {
                this["name"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the type of the service to use
        /// </summary>
        [ConfigurationProperty("type", IsRequired = true)]
        public Type ServiceType
        {
            get
            {
                return this["type"] == null ? null : Type.GetType((string)this["type"]);
            }

            set
            {
                this["type"] = value.ToString();
            }
        }

        /// <summary>
        /// Gets or sets whether or not the service is enabled
        /// </summary>
        [ConfigurationProperty("enabled", IsRequired = false, DefaultValue = true)]
        public bool Enabled
        {
            get
            {
                return (bool)this["enabled"];
            }

            set
            {
                this["enabled"] = value;
            }
        }
    }
}
