﻿//-----------------------------------------------------------------------
// <copyright file="Log.Data.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Data;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.General;

#endregion

namespace FastNET.CleanAct.Client.ManagerEngineData.Entities
{
    /// <summary>
    /// Log Entity Data Class
    /// </summary>
    internal partial class Log : ILogEntryData
    {
        private bool _isDirty;

        /// <summary>
        /// Initializes a new instance of the DynamicSetting class
        /// </summary>
        public Log()
        {
            LogID = Guid.NewGuid();
            _isDirty = false;
            PropertyChanged += EventHandler_PropertyChanged;
        }

        #region State Management

        private void EventHandler_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            _isDirty = true;
        }

        /// <summary>
        /// Gets a value indicating whether or not this instance has been modified
        /// </summary>
        public bool IsDirty
        {
            get
            {
                if (IsNew)
                {
                    return true;
                }

                return _isDirty;
            }

            internal set
            {
                _isDirty = false;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not this instance is new
        /// </summary>
        public bool IsNew
        {
            get
            {
                // Before it is attached if the key is null then we are new
                if (EntityState == EntityState.Detached)
                {
                    if (EntityKey == null)
                    {
                        return true;
                    }
                }

                return EntityState == EntityState.Added;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not this instance has been deleted
        /// </summary>
        public bool IsDeleted
        {
            get
            {
                return EntityState == EntityState.Deleted;
            }
        }

        #endregion

        #region Identity Management

        /// <summary>
        /// Gets an Identity based reference for this item
        /// </summary>
        public Identity ID
        {
            get { return LogID; }
        }

        #endregion

        #region Validations

        partial void OnAppDomainNameChanging(string value)
        {
            if (string.IsNullOrEmpty(value))
                throw new ArgumentNullException("value", "The AppDomainName value cannot be null or empty.");

            if (value.Length > 1000)
                throw new ArgumentOutOfRangeException("value", value, "The AppDomainName value specified is too long.");
        }

        partial void OnCategoriesChanging(string value)
        {
            if (!string.IsNullOrEmpty(value) && value.Length > 4000)
                throw new ArgumentOutOfRangeException("value", value, "The Categories value specified is too long.");
        }

        partial void OnMachineNameChanging(string value)
        {
            if (string.IsNullOrEmpty(value))
                throw new ArgumentNullException("value", "The MachineName value cannot be null or empty.");

            if (value.Length > 15)
                throw new ArgumentOutOfRangeException("value", value, "The MachineName value specified is too long.");
        }

        partial void OnMessageChanging(string value)
        {
            if (!string.IsNullOrEmpty(value) && value.Length > 4000)
                throw new ArgumentOutOfRangeException("value", value, "The Message value specified is too long.");
        }

        partial void OnProcessNameChanging(string value)
        {
            if (!string.IsNullOrEmpty(value) && value.Length > 1000)
                throw new ArgumentOutOfRangeException("value", value, "The ProcessName value specified is too long.");
        }

        partial void OnSeverityChanging(string value)
        {
            if (string.IsNullOrEmpty(value))
                throw new ArgumentNullException("value", "The Severity value cannot be null or empty.");

            if (value.Length > 100)
                throw new ArgumentOutOfRangeException("value", value, "The Severity value specified is too long.");
        }

        partial void OnThreadNameChanging(string value)
        {
            if (!string.IsNullOrEmpty(value) && value.Length > 1000)
                throw new ArgumentOutOfRangeException("value", value, "The ThreadName value specified is too long.");
        }

        partial void OnTitleChanging(string value)
        {
            if (string.IsNullOrEmpty(value))
                throw new ArgumentNullException("value", "The Title value cannot be null or empty.");

            if (value.Length > 200)
                throw new ArgumentOutOfRangeException("value", value, "The Title value specified is too long.");
        }

        #endregion
    }
}
