﻿//-----------------------------------------------------------------------
// <copyright file="TaskProcessor.cs" company="FastNET Services">
//     Copyright (c) 2007-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using System.Collections.Generic;
using FastNET.CleanAct.Client.Interfaces.TaskProcessing;

#endregion

namespace FastNET.CleanAct.Client.TaskProcessing.Basic
{
    /// <summary>
    /// Base class for task processors
    /// </summary>
    public abstract class TaskProcessor : ITaskProcessor
    {
        #region Variables

        private readonly List<Type> _compatibleTaskTypes;

        #endregion

        #region CTor/DTor

        /// <summary>
        /// Initializes a new instance of the TaskProcessor class
        /// </summary>
        protected TaskProcessor()
        {
            _compatibleTaskTypes = new List<Type>();
        }

        #endregion

        #region Public Collection Properties

        /// <summary>
        /// Gets a list of all the compatible types of tasks this processor can manage
        /// </summary>
        public IList<Type> CompatibleTaskTypes
        {
            get
            {
                return _compatibleTaskTypes;
            }
        }

        #endregion

        #region Public Abstract Properties

        /// <summary>
        /// Gets the name of the processor for reporting
        /// </summary>
        public abstract string ProcessorName { get; }

        /// <summary>
        /// Gets a description of the processor for reporting
        /// </summary>
        public abstract string ProcessorDescription { get; }

        #endregion
    }
}