﻿//-----------------------------------------------------------------------
// <copyright file="TriggerBase.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System.Security.Principal;
using FastNET.CleanAct.Client.Interfaces.Triggers;
#endregion

namespace FastNET.CleanAct.Client.TaskProcessing.Basic.Triggers
{
    /// <summary>
    /// Base class for all triggers
    /// </summary>
    public abstract class TriggerBase : ITrigger
    {
        /// <summary>
        /// Called when a user logs onto the system
        /// </summary>
        /// <param name="user">WindowsIdentity of the user the event that occured</param>
        public virtual void SessionLogon(WindowsIdentity user)
        {
        }

        /// <summary>
        /// Called when a user logs out of the system
        /// </summary>
        /// <param name="user">WindowsIdentity of the user the event that occured</param>
        public virtual void SessionLogoff(WindowsIdentity user)
        {
        }

        /// <summary>
        /// Called when a session is locked
        /// </summary>
        /// <param name="user">WindowsIdentity of the user the event that occured</param>
        public virtual void SessionLocked(WindowsIdentity user)
        {
        }

        /// <summary>
        /// Called when a session is unlocked
        /// </summary>
        /// <param name="user">WindowsIdentity of the user the event that occured</param>
        public virtual void SessionUnlocked(WindowsIdentity user)
        {
        }

        /// <summary>
        /// Called when the application starts
        /// </summary>
        public virtual void ApplicationStart()
        {
        }

        /// <summary>
        /// Called when the application stops
        /// </summary>
        public virtual void ApplicationStop()
        {
        }

        /// <summary>
        /// Called before the execution of the engine starts
        /// </summary>
        public virtual void EngineBeforeExecute()
        {
        }

        /// <summary>
        /// Called after the execution of the engine finishes
        /// </summary>
        public virtual void EngineAfterExecute()
        {
        }
    }
}
