//-----------------------------------------------------------------------
// <copyright file="ILogEntryData.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.Base;
#endregion

namespace FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.General
{
    /// <summary>
    /// Data interface for Log Entries
    /// </summary>
    public interface ILogEntryData : IIdentityData
    {
        /// <summary>
        /// Gets or sets the main message for the log
        /// </summary>
        string Message { get; set; }

        /// <summary>
        /// Gets or sets a comma separated list of categories for the log
        /// </summary>
        string Categories { get; set; }

        /// <summary>
        /// Gets or sets the priority for the log
        /// </summary>
        int Priority { get; set; }

        /// <summary>
        /// Gets or sets the specific code for the log to use in help documents
        /// </summary>
        int EventID { get; set; }

        /// <summary>
        /// Gets or sets a string for the severity of the log
        /// </summary>
        string Severity { get; set; }

        /// <summary>
        /// Gets or sets a brief title for the log
        /// </summary>
        string Title { get; set; }

        /// <summary>
        /// Gets or sets a DateTime the log event occurred
        /// </summary>
        DateTime TimeStamp { get; set; }

        /// <summary>
        /// Gets or sets the name of the computer the log occurred on
        /// </summary>
        string MachineName { get; set; }

        /// <summary>
        /// Gets or sets the name of the AppDomain the log occurred in
        /// </summary>
        string AppDomainName { get; set; }

        /// <summary>
        /// Gets or sets the ID for the process of the program
        /// </summary>
        int ProcessId { get; set; }

        /// <summary>
        /// Gets or sets the name of the process of the program
        /// </summary>
        string ProcessName { get; set; }

        /// <summary>
        /// Gets or sets the name of the thread of the program
        /// </summary>
        string ThreadName { get; set; }

        /// <summary>
        /// Gets or sets the ID for the thread of the program
        /// </summary>
        int? ThreadId { get; set; }

        /// <summary>
        /// Gets or sets an activity grouping ID for the log
        /// </summary>
        Guid? ActivityId { get; set; }

        /// <summary>
        /// Gets or sets the parent activity grouping for the log
        /// </summary>
        Guid? RelatedActivityId { get; set; }
    }
}