﻿//-----------------------------------------------------------------------
// <copyright file="IUserProfileRepository.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.Client;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.Base;
#endregion

namespace FastNET.CleanAct.Common.Repository.Interfaces.Repositories.Client
{
    /// <summary>
    /// Respository interface for managing IUserProfileData
    /// </summary>
    public interface IUserProfileRepository
        : IIdentityRepository<IUserProfileData>
    {
        /// <summary>
        /// Retrieves a single IUserProfileData by looking at the AccountName
        /// </summary>
        /// <param name="accountName">Account name to search by</param>
        /// <returns>IUserProfileData found or null if no account was found</returns>
        /// <exception cref="ArgumentNullException">Thrown if the search value is null or empty</exception>
        IUserProfileData RetrieveByAccountName(string accountName);

        /// <summary>
        /// Checks to see if an IUserProfileData exists by the AccountName
        /// </summary>
        /// <param name="accountName">Account name to search by</param>
        /// <param name="ignoreID">ID of an user profile to ignore during check</param>
        /// <returns>Boolean whether or not an user profile exists in the system</returns>
        /// <exception cref="ArgumentNullException">Thrown if AccountName is null or empty</exception>
        /// <exception cref="InvalidCastException">Thrown if IgnoreID is of the wrong type for this repository</exception>
        bool AccountNameExists(string accountName, Identity? ignoreID);
    }
}
