﻿//-----------------------------------------------------------------------
// <copyright file="ActionHelper.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.ManagementEngine;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.ManagementEngine;
using Action = FastNET.CleanAct.Client.ManagerEngineData.Entities.Action;

#endregion

namespace FastNET.CleanAct.Testing.Client.ManagerEngineData.Helpers
{
    /// <summary>
    /// Static Unit Test Helper Class for Action Entities
    /// </summary>
    internal static class ActionHelper
    {
        /// <summary>
        /// Creates a completely random and unique Action Type
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomActionType()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty).Substring(0, 20);
        }

        /// <summary>
        /// Creates a completely random and unique Name
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomName()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty);
        }

        /// <summary>
        /// Creates a completely random and unique Pattern
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomPattern()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty);
        }

        /// <summary>
        /// Creates an IActionData ready to save to the database
        /// </summary>
        /// <returns>IActionData with unique default values</returns>
        public static IActionData Create()
        {
            return Create(Guid.NewGuid());
        }

        /// <summary>
        /// Creates an IActionData ready to save to the database
        /// </summary>
        /// <param name="templateID">TemplateID to associate the item with</param>
        /// <returns>IActionData with unique default values</returns>
        public static IActionData Create(Identity templateID)
        {
            Action item = new Action();
            Random rnd = new Random();

            item.TemplateID = templateID;
            item.ActionType = CreateRandomActionType();
            item.Pattern = CreateRandomPattern();
            item.RecursiveMode = (byte)rnd.Next(0, 255);
            item.Name = CreateRandomName();

            return item;
        }

        /// <summary>
        /// Creates an IActionData ready pre-saved to the database
        /// </summary>
        /// <returns>IActionData with unique default values</returns>
        public static IActionData CreateDB()
        {
            return CreateDB(TemplateHelper.CreateDB().ID);
        }

        /// <summary>
        /// Creates an IActionData ready pre-saved to the database
        /// </summary>
        /// <param name="templateID">TemplateID to associate the character with</param>
        /// <returns>IActionData with unique default values</returns>
        public static IActionData CreateDB(Identity templateID)
        {
            Action item = (Action)Create(templateID);

            DBHelper.Provider.GetRepository<IActionRepository>().Save(item);

            return item;
        }
    }
}
