﻿//-----------------------------------------------------------------------
// <copyright file="LogHelper.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.General;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.General;
using Log = FastNET.CleanAct.Client.ManagerEngineData.Entities.Log;

#endregion

namespace FastNET.CleanAct.Testing.Client.ManagerEngineData.Helpers
{
    /// <summary>
    /// Static Unit Test Helper Class for Log Entities
    /// </summary>
    internal static class LogHelper
    {
        /// <summary>
        /// Creates a completely random and unique Name
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomString()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty).Substring(0, 10);
        }

        /// <summary>
        /// Creates an ILogEntryData ready to save to the database
        /// </summary>
        /// <returns>ILogEntryData with unique default values</returns>
        public static ILogEntryData Create()
        {
            Log item = new Log();
            Random rnd = new Random();

            item.AppDomainName = CreateRandomString();
            item.Categories = CreateRandomString();
            item.EventID = rnd.Next(0, Int32.MaxValue);
            item.MachineName = CreateRandomString();
            item.Priority = rnd.Next(0, Int32.MaxValue);
            item.ProcessId = rnd.Next(0, Int32.MaxValue);
            item.ProcessName = CreateRandomString();
            item.Severity = CreateRandomString();
            item.ThreadName = CreateRandomString();
            item.TimeStamp = DateTime.UtcNow;
            item.Title = CreateRandomString();

            return item;
        }

        /// <summary>
        /// Creates an ILogData ready pre-saved to the database
        /// </summary>
        /// <returns>ILogData with unique default values</returns>
        public static ILogEntryData CreateDB()
        {
            Log item = (Log)Create();

            DBHelper.Provider.GetRepository<ILogEntryRepository>().Save(item);

            return item;
        }
    }
}
