﻿//-----------------------------------------------------------------------
// <copyright file="ScheduleHelper.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.ManagementEngine;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.ManagementEngine;
using Schedule = FastNET.CleanAct.Client.ManagerEngineData.Entities.Schedule;

#endregion

namespace FastNET.CleanAct.Testing.Client.ManagerEngineData.Helpers
{
    /// <summary>
    /// Static Unit Test Helper Class for Schedule Entities
    /// </summary>
    internal static class ScheduleHelper
    {
        /// <summary>
        /// Creates a completely random and unique Name
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomName()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty).Substring(0, 20);
        }

        /// <summary>
        /// Creates a completely random and unique Value
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomValue()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty).Substring(0, 20);
        }

        /// <summary>
        /// Creates an IScheduleData ready to save to the database
        /// </summary>
        /// <returns>IScheduleData with unique default values</returns>
        public static IScheduleData Create()
        {
            return Create(Guid.NewGuid());
        }

        /// <summary>
        /// Creates an IScheduleData ready to save to the database
        /// </summary>
        /// <param name="templateID">TemplateID to associate the item with</param>
        /// <returns>IScheduleData with unique default values</returns>
        public static IScheduleData Create(Identity templateID)
        {
            Schedule item = new Schedule();
            Random rnd = new Random();

            item.TemplateID = templateID;
            item.CreatedDate = DateTime.UtcNow;
            item.Enabled = Convert.ToBoolean(rnd.Next(0, 1));
            item.ModifiedDate = DateTime.UtcNow;
            item.Name = CreateRandomName();
            item.Value = CreateRandomValue();

            return item;
        }

        /// <summary>
        /// Creates an IScheduleData ready pre-saved to the database
        /// </summary>
        /// <returns>IScheduleData with unique default values</returns>
        public static IScheduleData CreateDB()
        {
            return CreateDB(TemplateHelper.CreateDB().ID);
        }

        /// <summary>
        /// Creates an IScheduleData ready pre-saved to the database
        /// </summary>
        /// <param name="templateID">TemplateID to associate the character with</param>
        /// <returns>IScheduleData with unique default values</returns>
        public static IScheduleData CreateDB(Identity templateID)
        {
            Schedule item = (Schedule)Create(templateID);

            DBHelper.Provider.GetRepository<IScheduleRepository>().Save(item);

            return item;
        }
    }
}
