﻿//-----------------------------------------------------------------------
// <copyright file="TemplateHelper.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings

using System;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.ManagementEngine;
using FastNET.CleanAct.Common.Repository.Interfaces.Repositories.ManagementEngine;
using Template = FastNET.CleanAct.Client.ManagerEngineData.Entities.Template;

#endregion

namespace FastNET.CleanAct.Testing.Client.ManagerEngineData.Helpers
{
    /// <summary>
    /// Static Unit Test Helper Class for Template Entities
    /// </summary>
    internal static class TemplateHelper
    {
        /// <summary>
        /// Creates a completely random and unique Name
        /// </summary>
        /// <returns></returns>
        public static string CreateRandomName()
        {
            return Guid.NewGuid().ToString().Replace("-", string.Empty).Substring(0, 20);
        }

        /// <summary>
        /// Creates an ITemplateData ready to save to the database
        /// </summary>
        /// <returns>ITemplateData with unique default values</returns>
        public static ITemplateData Create()
        {
            return Create(null);
        }

        /// <summary>
        /// Creates an ITemplateData ready to save to the database
        /// </summary>
        /// <param name="packageID">Identity of the package the template is associated with</param>
        /// <returns>ITemplateData with unique default values</returns>
        public static ITemplateData Create(Identity? packageID)
        {
            Template item = new Template();
            Random rnd = new Random();

            item.IsBuiltIn = Convert.ToBoolean(rnd.Next(0, 1));
            item.Name = CreateRandomName();
            item.PackageID = packageID;

            return item;
        }

        /// <summary>
        /// Creates an ITemplateData ready pre-saved to the database
        /// </summary>
        /// <returns>ITemplateData with unique default values</returns>
        public static ITemplateData CreateDB()
        {
            return CreateDB(null);
        }

        /// <summary>
        /// Creates an ITemplateData ready pre-saved to the database
        /// </summary>
        /// <param name="packageID">Identity of the package the template is associated with</param>
        /// <returns>ITemplateData with unique default values</returns>
        public static ITemplateData CreateDB(Identity? packageID)
        {
            Template item = (Template)Create(packageID);

            DBHelper.Provider.GetRepository<ITemplateRepository>().Save(item);

            return item;
        }
    }
}
