﻿//-----------------------------------------------------------------------
// <copyright file="MockLog.cs" company="FastNET Services">
//     Copyright (c) 2008-2009. All Rights Reserved.
// </copyright>
// <license>
//     GPLv2
// </license>
// <warranty>
//     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
//     BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//     DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//     OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// </warranty>
//-----------------------------------------------------------------------
#region Usings
using System;
using FastNET.CleanAct.Common.Repository;
using FastNET.CleanAct.Common.Repository.Interfaces.DataInterfaces.General;
#endregion

namespace FastNET.CleanAct.Testing.Mockup.DataLayer
{
    /// <summary>
    /// Mock item for testing invalid repository objects
    /// </summary>
    public class MockLog : ILogEntryData
    {
        /// <summary>
        /// Gets a value indicating whether or not this instance has been changed
        /// </summary>
        public bool IsDirty { get; private set; }

        /// <summary>
        /// Gets a value indicating whether or not this instance is a new entity
        /// </summary>
        public bool IsNew { get; private set; }

        /// <summary>
        /// Gets a value indicating whether or not this instance has been deleted
        /// </summary>
        public bool IsDeleted { get; private set; }

        /// <summary>
        /// Gets the Identifier for the instance
        /// </summary>
        public Identity ID { get; private set; }

        /// <summary>
        /// Gets or sets the main message for the log
        /// </summary>
        public string Message { get; set; }

        /// <summary>
        /// Gets or sets a comma separated list of categories for the log
        /// </summary>
        public string Categories { get; set; }

        /// <summary>
        /// Gets or sets the priority for the log
        /// </summary>
        public int Priority { get; set; }

        /// <summary>
        /// Gets or sets the specific code for the log to use in help documents
        /// </summary>
        public int EventID { get; set; }

        /// <summary>
        /// Gets or sets a string for the severity of the log
        /// </summary>
        public string Severity { get; set; }

        /// <summary>
        /// Gets or sets a brief title for the log
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets a DateTime the log event occurred
        /// </summary>
        public DateTime TimeStamp { get; set; }

        /// <summary>
        /// Gets or sets the name of the computer the log occurred on
        /// </summary>
        public string MachineName { get; set; }

        /// <summary>
        /// Gets or sets the name of the AppDomain the log occurred in
        /// </summary>
        public string AppDomainName { get; set; }

        /// <summary>
        /// Gets or sets the ID for the process of the program
        /// </summary>
        public int ProcessId { get; set; }

        /// <summary>
        /// Gets or sets the name of the process of the program
        /// </summary>
        public string ProcessName { get; set; }

        /// <summary>
        /// Gets or sets the name of the thread of the program
        /// </summary>
        public string ThreadName { get; set; }

        /// <summary>
        /// Gets or sets the ID for the thread of the program
        /// </summary>
        public int? ThreadId { get; set; }

        /// <summary>
        /// Gets or sets an activity grouping ID for the log
        /// </summary>
        public Guid? ActivityId { get; set; }

        /// <summary>
        /// Gets or sets the parent activity grouping for the log
        /// </summary>
        public Guid? RelatedActivityId { get; set; }
    }
}
